
--[[ Use Addon Wide Data Table ]]--
local addonName,addonData = ...
addonData = addonData or {}

--[[ Generate a list of active frames for locating an available frame to use ]]--
addonData.GetActiveFrames = function(self)
	local shownList = {};
	local dockedList = {};
	local infoPanelList = {};
	local count = 0;
	local chatFrame;
	
	-- Cycle through ChatFrames 1 to 10 ( the current NUM_CHAT_WINDOWS setting )
	for i=1, NUM_CHAT_WINDOWS do

		-- Get the chatFrame object for this possible frame
		chatFrame = _G["ChatFrame"..i];
	
		-- Is it supposed to be shown ?
		local _, _, _, _, _, _, shown = FCF_GetChatWindowInfo(i);
				
		-- ChatFrame should be valid but double check
		if ( chatFrame ) then
		
			-- If it is flagged as a shown window then add it to the shown list
			if ( shown ) then shownList[chatFrame:GetID()] = true; end
			
			-- If it is flagged as a docked window then add it to the docked list
			if ( chatFrame.isDocked ) then dockedList[chatFrame:GetID()] = true; end

			-- If an info panel is using it then we don't want to use it
			if ( chatFrame.infoPanel ) then infoPanelList[chatFrame:GetID()] = true; end
			
		end
	end
	
	-- return the lists for viewing later
	return shownList,dockedList,infoPanelList;
end

--[[ Single function to decide what to put in the tab text ]]--
addonData.SetWindowName = function(self)
	if ( not addonData.ChatFrame ) then return end
	FCF_SetWindowName(addonData.ChatFrame, addonData.windowName);	
end

addonData.ExtractElements = function(self)
	-- Of we aren't ready to initialise then don't activate the plugin quite yet
	if ( not addonData.ChatFrame ) then return end

	addonData.Tab  			= _G[addonData.ChatFrame:GetName().."Tab"];
	addonData.Background	= _G[addonData.ChatFrame:GetName().."Background"];
	addonData.ButtonFrame	= _G[addonData.ChatFrame:GetName().."ButtonFrame"];
	addonData.EditBox		= _G[addonData.ChatFrame:GetName().."EditBox"];
	addonData.MinimizeButton= _G[addonData.ChatFrame:GetName().."MinimizeButton"];
    addonData.BottomButton = _G[addonData.ChatFrame:GetName()].ScrollToBottomButton;
end

--[[ Restore the frame we have been using ]]--
addonData.RestoreFrame = function(self,windowName)
	
	-- If we aren't ready to initialise then don't activate the plugin quite yet
	if ( not addonData.ChatFrame ) then return end

	-- Set the Window name which gets displayed on the tab
	self:SetWindowName(windowName);

	-- Make sure we have shortcut variables for the chat frame parts
	self:ExtractElements()
	
	addonData.ChatFrame.ipInitialised = false;
end

--[[ Initialise the chosen frame settings ready for use ]]--
addonData.InitialiseFrame = function(self,windowName)
	
	-- If we aren't ready to initialise then don't activate the plugin quite yet
	if ( not addonData.ChatFrame ) then return end

	-- Extract elements of the frame
	self:ExtractElements()

	-- Copy the settings from the default chat frame
	FCF_CopyChatSettings(addonData.ChatFrame, DEFAULT_CHAT_FRAME);

	-- Set the Window name which gets displayed on the tab
	self:SetWindowName(windowName);

	-- Clear any stale messages
	addonData.ChatFrame:Clear();

	--Clear the edit box history.
	addonData.EditBox:ClearHistory();

	addonData.ChatFrame.ipInitialised = true;
end

--[[ Disable the plugin when we need to ]]--
addonData.DisablePlugin = function(self,plugin)

	-- But if the frame doesn't exist it was never enabled *touch wood*
	if ( not addonData.ChatFrame ) then return end

	-- If we re-parented, which we should have, then put the parent back
	if plugin.chattab_parent then
		addonData.Tab:SetParent( plugin.chattab_parent );
		plugin.chattab_parent = nil;
	end

	-- If we re-parented the chatframe, and we definitely should have, then restore parent, re-dock, re-anchor
	if plugin.chatlog_parent then
		addonData.ChatFrame:SetParent( plugin.chatlog_parent );
		FCF_DockFrame( addonData.ChatFrame, #FCFDock_GetChatFrames(GENERAL_CHAT_DOCK)+1, true );
		plugin:SetScript( "OnUpdate", nil );
		plugin.chatlog_parent = nil;

		addonData.Background.SetPoint = addonData.Background.cachedSetPoint;
		addonData.Background:ClearAllPoints();
		addonData.Background:SetPoint( "TOPLEFT", addonData.ChatFrame, "TOPLEFT", -2, 3 );
		addonData.Background:SetPoint( "TOPRIGHT", addonData.ChatFrame, "TOPRIGHT", 2, 3 );
		addonData.Background:SetPoint( "BOTTOMLEFT", addonData.ChatFrame, "BOTTOMLEFT", -2, -6 );
		addonData.Background:SetPoint( "BOTTOMRIGHT", addonData.ChatFrame, "BOTTOMRIGHT", 3, -6 );
	end
	-- return with new values
	return plugin
end

--[[ Enable the plugin when we need to ]]--
addonData.EnablePlugin = function(self,plugin,windowName)

	-- Unless of course we try to before the frame has been generated
	if ( not addonData.ChatFrame ) then plugin.active = false; return end

	-- We're gonna re-parent so store the current parents
	plugin.chatlog_parent = addonData.ChatFrame:GetParent();
	plugin.chattab_parent = addonData.Tab:GetParent();

	-- If the frame/tab isn't shown then show them
	if not addonData.ChatFrame:IsShown() then
		addonData.ChatFrame:Show();
		addonData.Tab:Show();
		SetChatWindowShown(addonData.ChatFrame:GetID(), 1);
	end
	
	-- Start Moving/Sizing the frame
	MOVING_CHAT_FRAME = addonData.ChatFrame;

	-- Unlock the frame if it is locked
	FCF_SetLocked( addonData.ChatFrame, nil );
	
	-- Undock the frame if it is docked
	FCF_UnDockFrame( addonData.ChatFrame );
	
	-- Re-parent the chat tab and position it to the topleft of the chatframe
	addonData.Tab:SetParent(nil);
	addonData.Tab:ClearAllPoints();
	addonData.Tab:SetParent( plugin.container );
	addonData.Tab:SetPoint("BOTTOMLEFT",addonData.ChatFrame, "TOPLEFT",0,0);
	addonData.Tab:Hide();
	
	-- Reset the background of the chatframe to cover the whole frame
	addonData.Background:ClearAllPoints();
	addonData.Background:SetAllPoints( addonData.ChatFrame );
	
	-- The store and nullify the reanchoring function for the background
	addonData.Background.cachedSetPoint = addonData.Background.SetPoint;
	addonData.Background.SetPoint       = function() end;

    -- ButtonFrame needs to be stopped from moving by Blizz as it makes it go full screen stopping mouse access
    addonData.ButtonFrame.cachedSetPoint = addonData.ButtonFrame.SetPoint;
    addonData.ButtonFrame.SetPoint      = function() end;

	-- Carry out the actual Moving/Sizing of the chat frame
	addonData.ChatFrame:StartMoving();
	addonData.ChatFrame:StartSizing();
	addonData.ChatFrame:SetParent(nil);
	addonData.ChatFrame:ClearAllPoints();
	addonData.ChatFrame:SetParent( plugin.container );
	addonData.ChatFrame:SetPoint("TOPLEFT", plugin.container, "TOPLEFT",0,0);
	addonData.ChatFrame:SetPoint("BOTTOMRIGHT",plugin.container,"BOTTOMRIGHT",0,0);
	addonData.ChatFrame:StopMovingOrSizing();
	
	-- Okay, we're finished moving stuff
	MOVING_CHAT_FRAME = nil;
	
	-- Set some required values for future use
	addonData.ChatFrame:SetMovable( true );
	addonData.ChatFrame:SetResizable( true );
	addonData.ChatFrame:SetClampedToScreen( false );
	addonData.ChatFrame:SetUserPlaced( true );
	
	-- Locked the frame and turn off the backdrop color
	FCF_SetLocked( addonData.ChatFrame, 1 );
	FCF_SetWindowAlpha( addonData.ChatFrame, 0 );

	if ( addonData.ChatFrame.ipInitialised ) then
		-- First Remove all channels and message groups
		ChatFrame_RemoveAllChannels( addonData.ChatFrame );
		ChatFrame_RemoveAllMessageGroups( addonData.ChatFrame );
		ChatFrame_ReceiveAllPrivateMessages( addonData.ChatFrame );
		--ChatFrame_ReceiveAllBNConversations( addonData.ChatFrame );
	end
		
	-- Just in case Blizz overrode the tab text, restore the window name again
	self:SetWindowName(windowName);		               		               		

	-- Hide the minimize button as we shouldn't need to use it
	if ( addonData.MinimizeButton ) then RegisterStateDriver( addonData.MinimizeButton, "visibility", "hide" ); end
	
	-- Turn down the alpha setting of the other buttons
	addonData.BottomButton:SetAlpha( 0 );			

	-- set method to show or hide the scroll buttons based on whether or
	-- not the mouse is in the frame
	
	addonData.ChatFrame.showButtons = function( enabled )		
		if enabled ~= addonData.ChatFrame.buttons_visible then					
			addonData.ChatFrame.buttons_visible = enabled;
			addonData.BottomButton:SetAlpha( enabled and 1 or 0 );
		end
	end
	
	-- allow scrolling of the chat log frame with the mouse wheel	
	addonData.ChatFrame:EnableMouseWheel( true );	
	
	addonData.ChatFrame:SetScript( "OnMouseWheel", 
		function()
			if arg1 > 0 then addonData.ChatFrame:ScrollUp();
			else addonData.ChatFrame:ScrollDown();
			end	
		end
	);	

	-- Now that we're done here show the frame and tab
	addonData.ChatFrame:Show();
	addonData.Tab:Show();
	SetChatWindowShown(addonData.ChatFrame:GetID(), 1);

	-- return with updated info
	return plugin
end


